@@ -4,6 +4,7 @@ Requires cx_Oracle: http://www.python.net/crew/atuining/cx_Oracle/
 """
 
 import os, sys
+import thread
 from django.db.backends import BaseDatabaseWrapper, BaseDatabaseValidation, util
 try:
     from django.db.backends.signals import connection_created
@@ -57,6 +58,7 @@ def get_extras(database='default'):
                       'log':0,         # extra logging functionality
                       'logpath':'',    # file system path for oraclepool.log file
                       'existing':'',   # Type modifications if using existing database data
+                      'like':'LIKEC',  # Use LIKE or LIKEC - Oracle ignores index for LIKEC on older dbs
                       'session':[]     # Add session optimisations applied to each fresh connection, eg.
                                        #   ['alter session set cursor_sharing = similar',
                                        #    'alter session set session_cached_cursors = 20']
@@ -96,7 +98,7 @@ def get_logger(extras):
             logfile = os.path.join(os.path.abspath(os.path.dirname(logfile)),filename)
         # if log file is writable do it
         if not logfile:
-            raise 'Log path %s not found' % extras.get('logpath','')
+            raise Exception('Log path %s not found' % extras.get('logpath',''))
             return None
         else:
             logging.basicConfig(filename=logfile, level=loglevel)
@@ -180,7 +182,11 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             version - tested with both 1.0 and 1.2
         """
         super(DatabaseWrapper, self).__init__(*args, **kwargs)
-
+        if DATABASE_EXTRAS.get('like', 'LIKEC') != 'LIKEC':
+            for key in ['contains','icontains',
+                         'startswith','istartswith',
+                         'endswith','iendswith']:
+                self.operators[key] = self.operators[key].replace('LIKEC',DATABASE_EXTRAS['like'])
         self.features = DatabaseFeatures()
         self.ops = OracleDatabaseOperations()
         self.client = OracleDatabaseClient(self)
@@ -208,54 +214,61 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         return self.poolprops
 
     def _get_pool (self):
-        """ Get the connection pool or create it if it doesnt exist """
+        """ Get the connection pool or create it if it doesnt exist
+            Add thread lock to prevent server initial heavy load creating multiple pools
+        """
         if not hasattr (self.__class__, '_pool'):
-            if DATABASE_EXTRAS['threaded']:
-                Database.OPT_Threading = 1
-            else:
-                Database.OPT_Threading = 0
-            # Use 1.2 style dict if its there, else make one
-            try:
-                settings_dict = self.creation.connection.settings_dict
-            except:
-                settings_dict = None
-            if not settings_dict:
-                settings_dict = {'HOST':settings.DATABASE_HOST,
-                                 'PORT':settings.DATABASE_PORT,
-                                 'NAME':settings.DATABASE_NAME,
-                                 'USER':settings.DATABASE_USER, 
-                                 'PASSWORD':settings.DATABASE_PASSWORD, 
-                                 }
-            if len(settings_dict.get('HOST','').strip()) == 0:
-                settings_dict['HOST'] = 'localhost'
-            if len(settings_dict.get('PORT','').strip()) != 0:
-                dsn = Database.makedsn(settings_dict['HOST'],
-                                       int(settings_dict['PORT']),
-                                       settings_dict.get('NAME',''))
-            else:
-                dsn = settings_dict.get('NAME','')
+            lock = thread.allocate_lock()
+            lock.acquire()
+            if not hasattr (self.__class__, '_pool'):            
+                if DATABASE_EXTRAS['threaded']:
+                    Database.OPT_Threading = 1
+                else:
+                    Database.OPT_Threading = 0
+                # Use 1.2 style dict if its there, else make one
+                try:
+                    settings_dict = self.creation.connection.settings_dict
+                except:
+                    settings_dict = None
 
-            try:
-                p = Database.SessionPool(settings_dict.get('USER',''), 
-                                         settings_dict.get('PASSWORD',''), 
-                                         dsn, 
-                                         DATABASE_EXTRAS.get('min',4), 
-                                         DATABASE_EXTRAS.get('max',8), 
-                                         DATABASE_EXTRAS.get('increment',1),
-                                         threaded = DATABASE_EXTRAS.get('threaded',True))
-            except Exception, err:
-                p = None
-            if p:
-                if DATABASE_EXTRAS.get('timeout',0):
-                    p.timeout = DATABASE_EXTRAS['timeout']
-                setattr(self.__class__, '_pool', p)
-            else:
-                msg = """##### Database '%s' login failed or database not found ##### 
-                         Using settings: %s 
-                         Django start up cancelled
-                      """ % (settings_dict.get('NAME','None'), settings_dict)
-                print msg
-                print '\n##### DUE TO ERROR: %s\n' % err
+                if not settings_dict.get('NAME',''):
+                    settings_dict = {'HOST':settings.DATABASE_HOST,
+                                     'PORT':settings.DATABASE_PORT,
+                                     'NAME':settings.DATABASE_NAME,
+                                     'USER':settings.DATABASE_USER, 
+                                     'PASSWORD':settings.DATABASE_PASSWORD, 
+                                     }
+                if len(settings_dict.get('HOST','').strip()) == 0:
+                    settings_dict['HOST'] = 'localhost'
+                if len(settings_dict.get('PORT','').strip()) != 0:
+                    dsn = Database.makedsn(settings_dict['HOST'],
+                                           int(settings_dict['PORT']),
+                                           settings_dict.get('NAME',''))
+                else:
+                    dsn = settings_dict.get('NAME','')
+
+                try:
+                    p = Database.SessionPool(settings_dict.get('USER',''), 
+                                             settings_dict.get('PASSWORD',''), 
+                                             dsn, 
+                                             DATABASE_EXTRAS.get('min',4), 
+                                             DATABASE_EXTRAS.get('max',8), 
+                                             DATABASE_EXTRAS.get('increment',1),
+                                             threaded = DATABASE_EXTRAS.get('threaded',True))
+                except Exception, err:
+                    p = None
+                if p:
+                    if DATABASE_EXTRAS.get('timeout',0):
+                        p.timeout = DATABASE_EXTRAS['timeout']
+                    setattr(self.__class__, '_pool', p)
+                else:
+                    msg = """##### Database '%s' login failed or database not found ##### 
+                             Using settings: %s 
+                             Django start up cancelled
+                          """ % (settings_dict.get('NAME','None'), settings_dict)
+                    print msg
+                    print '\n##### DUE TO ERROR: %s\n' % err
+                lock.release()
         return getattr(self.__class__, '_pool')
         
     pool = property (_get_pool)
@@ -305,7 +318,7 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             if logger:
                 logger.critical('Pool couldnt be created')
             else:
-                raise 'Pool couldnt be created'
+                raise Exception('Pool couldnt be created')
             
         if not cursor:
             cursor = FormatStylePlaceholderCursor(self.connection)
@@ -351,7 +364,7 @@ class FormatStylePlaceholderCursor(OracleFormatStylePlaceholderCursor):
                 if logger:
                     logger.critical(err)
                 else:
-                    raise str(err)
+                    raise Exception(err)
                 
     def execute(self, query, params=[]):
         if params is None:
@@ -371,7 +384,7 @@ class FormatStylePlaceholderCursor(OracleFormatStylePlaceholderCursor):
             if logger:
                 logger.critical(err)
             else:
-                raise str(err)
+                raise 
 
     def executemany(self, query, params=[]):
         try:
@@ -392,4 +405,4 @@ class FormatStylePlaceholderCursor(OracleFormatStylePlaceholderCursor):
             if logger:
                 logger.critical('%s due to query:%s' % (e, query))                
             else:
-                raise str(e)
+                raise 
